@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
+# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
 # SPDX-License-Identifier: Apache-2.0
 # fmt: off
 # isort: skip_file
@@ -7,6 +7,7 @@ from .value_pb2 import Enum, GenMap, Identifier, List, Map, Optional, Record, Re
 from .commands_pb2 import Command, Commands, CreateAndExerciseCommand, CreateCommand, ExerciseByKeyCommand, ExerciseCommand
 from .completion_pb2 import Completion
 from .event_pb2 import ArchivedEvent, CreatedEvent, Event, ExercisedEvent
+from .experimental_features_pb2 import ExperimentalFeatures, ExperimentalSelfServiceErrorCodes
 from .ledger_offset_pb2 import LedgerOffset
 from .transaction_pb2 import Transaction, TransactionTree, TreeEvent
 from .transaction_filter_pb2 import Filters, InclusiveFilters, TransactionFilter
@@ -26,7 +27,7 @@ from .package_service_pb2 import GetPackageRequest, GetPackageResponse, GetPacka
 from .package_service_pb2_grpc import PackageServiceStub
 from .transaction_service_pb2 import GetFlatTransactionResponse, GetLedgerEndRequest, GetLedgerEndResponse, GetTransactionByEventIdRequest, GetTransactionByIdRequest, GetTransactionResponse, GetTransactionTreesResponse, GetTransactionsRequest, GetTransactionsResponse
 from .transaction_service_pb2_grpc import TransactionServiceStub
-from .version_service_pb2 import GetLedgerApiVersionRequest, GetLedgerApiVersionResponse
+from .version_service_pb2 import CommandDeduplicationFeatures, DeduplicationPeriodSupport, FeaturesDescriptor, GetLedgerApiVersionRequest, GetLedgerApiVersionResponse, ParticipantDeduplicationSupport, UserManagementFeature
 from .version_service_pb2_grpc import VersionServiceStub
 
 __all__ = [
@@ -35,6 +36,7 @@ __all__ = [
     "Checkpoint",
     "Command",
     "CommandCompletionServiceStub",
+    "CommandDeduplicationFeatures",
     "CommandServiceStub",
     "CommandSubmissionServiceStub",
     "Commands",
@@ -46,11 +48,15 @@ __all__ = [
     "CreateAndExerciseCommand",
     "CreateCommand",
     "CreatedEvent",
+    "DeduplicationPeriodSupport",
     "Enum",
     "Event",
     "ExerciseByKeyCommand",
     "ExerciseCommand",
     "ExercisedEvent",
+    "ExperimentalFeatures",
+    "ExperimentalSelfServiceErrorCodes",
+    "FeaturesDescriptor",
     "Filters",
     "GenMap",
     "GetActiveContractsRequest",
@@ -88,6 +94,7 @@ __all__ = [
     "Optional",
     "PackageServiceStub",
     "PackageStatus",
+    "ParticipantDeduplicationSupport",
     "Record",
     "RecordField",
     "SubmitAndWaitForTransactionIdResponse",
@@ -100,6 +107,7 @@ __all__ = [
     "TransactionServiceStub",
     "TransactionTree",
     "TreeEvent",
+    "UserManagementFeature",
     "Value",
     "Variant",
     "VersionServiceStub",
