@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
+# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
 # SPDX-License-Identifier: Apache-2.0
 """
 :mod:`dazl.ledger.blocking`
@@ -21,7 +21,20 @@ else:
     from typing_extensions import Protocol, runtime_checkable
 
 
-__all__ = ["Connection", "QueryStream"]
+__all__ = ["PackageService", "Connection", "QueryStream", "PackageLoader"]
+
+
+class PackageService(Protocol):
+    """
+    Protocol that describe a service that provides package information. The :class:`Connection`
+    protocol extends this interface.
+    """
+
+    async def get_package(self, __package_id) -> bytes:
+        raise NotImplementedError
+
+    async def list_package_ids(self):
+        raise NotImplementedError
 
 
 @runtime_checkable
@@ -56,3 +69,7 @@ class QueryStream(_QueryStream, Protocol):
 
     def __iter__(self):
         raise NotImplementedError
+
+
+# Imports internal to this package are at the bottom of the file to avoid circular dependencies
+from .pkgloader import PackageLoader  # noqa
