@@ -1,5 +1,6 @@
-# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
+# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
 # SPDX-License-Identifier: Apache-2.0
+
 from typing import List, Tuple, Union, cast
 from urllib.parse import urlparse
 
@@ -66,6 +67,7 @@ class GrpcAuth(AuthMetadataPlugin):
         # TODO: Add support here for refresh tokens
         token = self._config.access.token
         if token:
-            options.append(("Authorization", "Bearer " + self._config.access.token))
+            # note: gRPC headers MUST be lowercased
+            options.append(("authorization", "Bearer " + self._config.access.token))
 
         callback(tuple(options), None)
